@@ -34,6 +34,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <inttypes.h>
+#include <sys/types.h>
 
 #ifdef __AVX
 #define BYTE_ALIGNMENT 32
@@ -50,6 +51,8 @@
 
 #define NUM_RELL_BOOTSTRAPS 1000
 
+#define NUM_ASC_CORRECTIONS 6
+
 #define MAX_TIP_EV     0.999999999 /* max tip vector value, sum of EVs needs to be smaller than 1.0, otherwise the numerics break down */
 #define smoothings     32          /* maximum smoothing passes through tree */
 #define iterations     10          /* maximum iterations of iterations per insert */
@@ -82,7 +85,7 @@
 
 #define NUM_BRANCHES   128
 
-#define TRUE             1
+#define TRUE            1
 #define FALSE            0
 
 
@@ -165,8 +168,9 @@
 #define PointGamma(prob,alpha,beta)  PointChi2(prob,2.0*(alpha))/(2.0*(beta))
 
 #define programName        "RAxML"
-#define programVersion     "8.1.1"
-#define programDate        "July 31 2014"
+#define programVersion     "8.2.0"
+#define programVersionInt   8200
+#define programDate        "July 9 2015"
 
 
 #define  TREE_EVALUATION                 0
@@ -198,7 +202,13 @@
 #define  PLAUSIBILITY_CHECKER            26
 #define  CALC_BIPARTITIONS_IC            27
 #define  ROOT_TREE                       28
-#define  MISSING_SEQUENCE_PREDICTION     29
+#define  STEAL_BRANCH_LENGTHS            29
+#define  SUBTREE_EPA                     30
+
+#define AUTO_ML   0
+#define AUTO_BIC  1
+#define AUTO_AIC  2
+#define AUTO_AICC 3
 
 #define M_GTRCAT         1
 #define M_GTRGAMMA       2
@@ -229,16 +239,17 @@
 #define HIVW         15
 #define JTTDCMUT     16
 #define FLU          17 
-#define DUMMY        18
-#define DUMMY2       19
-#define AUTO         20
-#define LG4          21
-#define LG4X         22
-#define PROT_FILE    23
-#define GTR_UNLINKED 24
-#define GTR          25  /* GTR always needs to be the last one */
+#define STMTREV      18
+#define DUMMY        19
+#define DUMMY2       20
+#define AUTO         21
+#define LG4          22
+#define LG4X         23
+#define PROT_FILE    24
+#define GTR_UNLINKED 25
+#define GTR          26  /* GTR always needs to be the last one */
 
-#define NUM_PROT_MODELS 26
+#define NUM_PROT_MODELS 27
 
 
 
@@ -355,8 +366,12 @@ struct ent
   
   //added by Kassian for TC/IC correction on partial gene trees
   unsigned int *taxonMask;
-  boolean wasFound;
-  unsigned int bLink;
+  unsigned int   bLink;
+  double         adjustedSupport;
+  double         tempSupport;
+  int            tempSupportFrom;
+  unsigned int   coveredNumber;
+  boolean        covered;
   //Kassian modif end 
 
   struct ent *next;
@@ -424,6 +439,11 @@ typedef struct
   int tipCase;
 #ifdef _HET
   boolean parentIsTip;
+#endif
+#ifdef _BASTIEN
+  double secondDerivativeQ[NUM_BRANCHES];
+  double secondDerivativeR[NUM_BRANCHES];
+  double secondDerivativeP[NUM_BRANCHES];
 #endif
   int pNumber;
   int qNumber;
@@ -501,6 +521,10 @@ typedef  struct noderec
 {  
   branchInfo      *bInf;
   double           z[NUM_BRANCHES];
+#ifdef _BASTIEN
+  double           secondDerivative[NUM_BRANCHES];
+  boolean          secondDerivativeValid[NUM_BRANCHES];
+#endif
   struct noderec  *next;
   struct noderec  *back;
   hashNumberType   hash;
@@ -580,13 +604,16 @@ typedef struct {
   boolean ascBias;  
   int     ascOffset;
   int     *ascExpVector;
+  int     *ascMissingVector;
   double  *ascSumBuffer;
   double  *ascVector;
+  double ascScaler[64];
   //asc bias end
 
 
   char   *partitionName;
   char   proteinSubstitutionFileName[2048];
+  char   ascFileName[2048];
   double externalAAMatrix[420];
 
   double *sumBuffer;
@@ -599,6 +626,9 @@ typedef struct {
   double *left;
   double *right;
 
+  double    *invariableFrequencies;
+  double    invariableWeight;
+
 #ifdef _HET
   /* heterotachy */
  
@@ -614,6 +644,7 @@ typedef struct {
   /* LG4 */
 
   double *EIGN_LG4[4];
+  double *rawEIGN_LG4[4];
   double *EV_LG4[4];
   double *EI_LG4[4];  
 
@@ -756,6 +787,16 @@ typedef  struct
 
 
 
+/***********************************************************/
+
+#define NOT_DEFINED              0
+#define LEWIS_CORRECTION         1
+#define FELSENSTEIN_CORRECTION   2
+#define STAMATAKIS_CORRECTION    3
+#define GOLDMAN_CORRECTION_1     4
+#define GOLDMAN_CORRECTION_2     5
+#define GOLDMAN_CORRECTION_3     6
+
 /**************************************************************/
 
 
@@ -840,14 +881,9 @@ typedef  struct  {
      not change depending on datatype */
 
   double           *invariants;
-  double           *fracchanges;
  
-  double           *rawFracchanges;
 
-#ifdef _HET
-  double *fracchanges_TIP;
-  double *rawFracchanges_TIP;
-#endif
+
 
   /* model stuff end */
 
@@ -862,16 +898,23 @@ typedef  struct  {
   int              *secondaryStructurePairs;
 
 
-  double            *partitionContributions;
+  double            *partitionContributions; 
 
-  double            fracchange;
-  double            rawFracchange;
+  int               ascertainmentCorrectionType;
+  int               autoProteinSelectionType;
 
-#ifdef _HET
-  double            fracchange_TIP;
-  double            rawFracchange_TIP;
+  unsigned int      numberOfEPAEntries;
+  double            accumulatedEPACutoff;
+  boolean           useAccumulatedEPACutoff;
+  double            probThresholdEPA;
+
+#ifdef _BASTIEN
+  double           secondDerivative[NUM_BRANCHES];
+  boolean          doBastienStuff;
 #endif
 
+
+
   double            lhCutoff;
   double            lhAVG;
   uint64_t          lhDEC;
@@ -896,6 +939,7 @@ typedef  struct  {
   boolean          searchConvergenceCriterion;
   int              branchLabelCounter;
   int              ntips;
+  int              binaryFile_ntips;
   int              nextnode;
   int              NumberOfModels;
   int              parsimonyLength;
@@ -951,6 +995,10 @@ typedef  struct  {
 
   boolean corrected_IC_Score;
 
+  boolean useK80;
+  boolean useHKY85;
+  boolean useJC69;
+
 #ifdef _USE_PTHREADS
 
   double *ancestralStates;
@@ -1017,8 +1065,11 @@ typedef  struct  {
   pthread_mutex_t** mutexesForHashing; 
 
 #endif
-  
-  int *origNumSitePerModel;
+    
+  boolean doSubtreeEPA;
+
+  double ascMissing;
+  boolean useAscMissing;
 
 } tree;
 
@@ -1126,6 +1177,7 @@ typedef  struct {
   boolean       silent;
   boolean       noSequenceCheck;
   boolean       useBFGS;
+  boolean       setThreadAffinity;
 } analdef;
 
 
@@ -1158,7 +1210,7 @@ typedef struct
 
 /****************************** FUNCTIONS ****************************************************/
 
-extern void ascertainmentBiasSequence(unsigned char tip[32], int numStates);
+extern void ascertainmentBiasSequence(unsigned char tip[32], int numStates, int dataType, int nodeNumber, int *ascMissingVector);
 
 extern void computePlacementBias(tree *tr, analdef *adef);
 
@@ -1210,15 +1262,19 @@ extern double LnGamma ( double alpha );
 extern double IncompleteGamma ( double x, double alpha, double ln_gamma_alpha );
 extern double PointNormal ( double prob );
 extern double PointChi2 ( double prob, double v );
-extern void makeGammaCats (double alpha, double *gammaRates, int K,  boolean useMedian);
+extern void makeGammaCats (int rateHetModel, double alpha, double *gammaRates, int K,  boolean useMedian, double propInvariant);
 extern void initModel ( tree *tr, rawdata *rdta, cruncheddata *cdta, analdef *adef );
 extern void doAllInOne ( tree *tr, analdef *adef );
 
-extern void classifyML(tree *tr, analdef *adef);
+extern void classifyML(tree *tr, analdef *adef) __attribute__((noreturn));
 extern void classifyMP(tree *tr, analdef *adef);
+extern void subtreeEPA(tree *tr, analdef *adef) __attribute__((noreturn));
+extern void collectSubtrees(tree *tr, nodeptr *subtrees, int *count, int ogn);
+extern int treeFindTipByLabelString(char  *str, tree *tr, boolean check);
+extern ssize_t rax_getline(char **lineptr, size_t *n, FILE *h);
 extern void markTips(nodeptr p, int *perm, int maxTips);
 extern char *Tree2StringClassify(char *treestr, tree *tr, int *inserts, 
-				 boolean  originalTree, boolean jointLabels, boolean likelihood);
+				 boolean  originalTree, boolean jointLabels, boolean likelihood, int rootNumber, boolean subtreePlacement);
 
 
 extern void doBootstrap ( tree *tr, analdef *adef, rawdata *rdta, cruncheddata *cdta );
@@ -1333,7 +1389,7 @@ extern double evaluateGenericVector (tree *tr, nodeptr p);
 extern void categorizeGeneric (tree *tr, nodeptr p);
 extern double makenewzPartitionGeneric(tree *tr, nodeptr p, nodeptr q, double z0, int maxiter, int model);
 extern boolean isTip(int number, int maxTips);
-extern void computeTraversalInfo(nodeptr p, traversalInfo *ti, int *counter, int maxTips, int numBranches);
+extern void computeTraversalInfo(tree *tr, nodeptr p, traversalInfo *ti, int *counter, int maxTips, int numBranches);
 
 
 
@@ -1385,7 +1441,7 @@ extern nodeptr findAnyTip(nodeptr p, int numsp);
 
 extern void parseProteinModel(double *externalAAMatrix, char *fileName);
 extern int filexists(char *filename);
-extern void computeFullTraversalInfo(nodeptr p, traversalInfo *ti, int *counter, int maxTips, int numBranches);
+extern void computeFullTraversalInfo(tree *tr, nodeptr p, traversalInfo *ti, int *counter, int maxTips, int numBranches);
 
 extern void computeNextReplicate(tree *tr, int64_t *seed, int *originalRateCategories, int *originalInvariant, boolean isRapid, boolean fixRates, analdef *adef);
 /*extern void computeNextReplicate(tree *tr, analdef *adef, int *originalRateCategories, int *originalInvariant);*/
@@ -1443,7 +1499,8 @@ extern void newviewIterativeAncestral(tree *tr);
 extern void newviewGenericAncestral(tree *tr, nodeptr p, boolean atRoot);
 extern void computeAncestralStates(tree *tr, double referenceLikelihood);
 extern void makeP_Flex(double z1, double z2, double *rptr, double *EI,  double *EIGN, int numberOfCategories, double *left, double *right, const int numStates);
-
+extern void makeP_FlexLG4(double z1, double z2, double *rptr, double *EI[4],  double *EIGN[4], int numberOfCategories, double *left, double *right, const int numStates);
+extern void scaleLG4X_EIGN(tree *tr, int model);
 extern void *rax_malloc( size_t size );
 extern void *rax_realloc(void *p, size_t size, boolean needsMemoryAlignment);
 extern void rax_free(void *p);
@@ -1527,6 +1584,7 @@ extern void testInsertThoroughIterative(tree *tr, int branchNumber);
 #define THREAD_OPT_LG4X_RATES               45
 #define THREAD_FREE_VECTORS                 46
 #define THREAD_SETUP_PRESENCE_MAP           47
+#define THREAD_COPY_LG4X_EIGN               48
 
 
 /*
